@@ -1,8 +1,8 @@
 import numpy as np
 from numpy.compat import basestring
 
-from ..utils import derived_from
-from .core import asarray, blockwise, einsum_lookup
+from dask.array.core import asarray, blockwise, einsum_lookup
+from dask.utils import derived_from
 
 einsum_symbols = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
 einsum_symbols_set = set(einsum_symbols)
@@ -194,10 +194,14 @@ def parse_einsum_input(operands):
 
 
 @derived_from(np)
-def einsum(*operands, **kwargs):
-    dtype = kwargs.pop("dtype", None)
-    optimize = kwargs.pop("optimize", False)
-    split_every = kwargs.pop("split_every", None)
+def einsum(*operands, dtype=None, optimize=False, split_every=None, **kwargs):
+    """Dask added an additional keyword-only argument ``split_every``.
+
+    split_every: int >= 2 or dict(axis: int), optional
+        Determines the depth of the recursive aggregation.
+        Deafults to ``None`` which would let dask heuristically
+        decide a good default.
+    """
 
     einsum_dtype = dtype
 
