@@ -9,10 +9,9 @@ try:
 except ImportError:
     scipy = None
 
-from .core import concatenate as _concatenate
-from .creation import arange as _arange
-from ..utils import derived_from, skip_doctest
-
+from dask.array.core import concatenate as _concatenate
+from dask.array.creation import arange as _arange
+from dask.utils import derived_from, skip_doctest
 
 chunk_error = (
     "Dask array only supports taking an FFT along an axis that \n"
@@ -33,7 +32,7 @@ fft_preamble = """
 
 
 def _fft_out_chunks(a, s, axes):
-    """ For computing the output chunks of [i]fft*"""
+    """For computing the output chunks of [i]fft*"""
     if s is None:
         return a.chunks
     chunks = list(a.chunks)
@@ -43,7 +42,7 @@ def _fft_out_chunks(a, s, axes):
 
 
 def _rfft_out_chunks(a, s, axes):
-    """ For computing the output chunks of rfft*"""
+    """For computing the output chunks of rfft*"""
     if s is None:
         s = [a.chunks[axis][0] for axis in axes]
     s = list(s)
@@ -55,7 +54,7 @@ def _rfft_out_chunks(a, s, axes):
 
 
 def _irfft_out_chunks(a, s, axes):
-    """ For computing the output chunks of irfft*"""
+    """For computing the output chunks of irfft*"""
     if s is None:
         s = [a.chunks[axis][0] for axis in axes]
         s[-1] = 2 * (s[-1] - 1)
@@ -194,7 +193,7 @@ def fft_wrap(fft_func, kind=None, dtype=None):
     if kind.endswith("fft"):
         _func = func
 
-        def func(a, n=None, axis=None):
+        def func(a, n=None, axis=None):  # type: ignore
             s = None
             if n is not None:
                 s = (n,)
