@@ -1,3 +1,5 @@
+import sys
+
 import pytest
 
 pytest.importorskip("numpy")
@@ -5,9 +7,11 @@ pytest.importorskip("scipy")
 
 import numpy as np
 import scipy.linalg
+from packaging.version import parse as parse_version
 
 import dask.array as da
 from dask.array.linalg import qr, sfqr, svd, svd_compressed, tsqr
+from dask.array.numpy_compat import _np_version
 from dask.array.utils import assert_eq, same_keys, svd_flip
 
 
@@ -449,14 +453,14 @@ def test_dask_svd_self_consistent(m, n):
         assert d_e.dtype == e.dtype
 
 
-@pytest.mark.parametrize("iterator", [("power", 1), ("QR", 1)])
+@pytest.mark.parametrize("iterator", ["power", "QR"])
 def test_svd_compressed_compute(iterator):
     x = da.ones((100, 100), chunks=(10, 10))
     u, s, v = da.linalg.svd_compressed(
-        x, k=2, iterator=iterator[0], n_power_iter=iterator[1], compute=True, seed=123
+        x, k=2, iterator=iterator, n_power_iter=1, compute=True, seed=123
     )
     uu, ss, vv = da.linalg.svd_compressed(
-        x, k=2, iterator=iterator[0], n_power_iter=iterator[1], seed=123
+        x, k=2, iterator=iterator, n_power_iter=1, seed=123
     )
 
     assert len(v.dask) < len(vv.dask)
@@ -532,20 +536,6 @@ def test_svd_compressed_shapes(m, n, k, chunks):
     assert v.shape == (r, n)
 
 
-@pytest.mark.parametrize("iterator", [("power", 1), ("QR", 1)])
-def test_svd_compressed_compute(iterator):
-    x = da.ones((100, 100), chunks=(10, 10))
-    u, s, v = da.linalg.svd_compressed(
-        x, 2, iterator=iterator[0], n_power_iter=iterator[1], compute=True, seed=123
-    )
-    uu, ss, vv = da.linalg.svd_compressed(
-        x, 2, iterator=iterator[0], n_power_iter=iterator[1], seed=123
-    )
-
-    assert len(v.dask) < len(vv.dask)
-    assert_eq(v, vv)
-
-
 def _check_lu_result(p, l, u, A):
     assert np.allclose(p.dot(l).dot(u), A)
 
@@ -976,6 +966,11 @@ def test_svd_incompatible_dimensions(ndim):
         da.linalg.svd(x)
 
 
+@pytest.mark.xfail(
+    sys.platform == "darwin" and _np_version < parse_version("1.22"),
+    reason="https://github.com/dask/dask/issues/7189",
+    strict=False,
+)
 @pytest.mark.parametrize(
     "shape, chunks, axis",
     [[(5,), (2,), None], [(5,), (2,), 0], [(5,), (2,), (0,)], [(5, 6), (2, 2), None]],
@@ -993,6 +988,11 @@ def test_norm_any_ndim(shape, chunks, axis, norm, keepdims):
 
 
 @pytest.mark.slow
+@pytest.mark.xfail(
+    sys.platform == "darwin" and _np_version < parse_version("1.22"),
+    reason="https://github.com/dask/dask/issues/7189",
+    strict=False,
+)
 @pytest.mark.parametrize(
     "shape, chunks",
     [
