@@ -1,6 +1,6 @@
-from itertools import zip_longest
 import os
 import warnings
+from itertools import permutations, zip_longest
 
 import pytest
 
@@ -9,13 +9,9 @@ np = pytest.importorskip("numpy")
 import itertools
 
 import dask.array as da
-from dask.array.utils import assert_eq as _assert_eq, same_keys
-from dask.core import get_deps
 import dask.config as config
-
-
-def assert_eq(a, b):
-    _assert_eq(a, b, equal_nan=True)
+from dask.array.utils import assert_eq, same_keys
+from dask.core import get_deps
 
 
 @pytest.mark.parametrize("dtype", ["f4", "i4"])
@@ -48,6 +44,38 @@ def test_numel(dtype, keepdims):
             )
 
 
+def reduction_0d_test(da_func, darr, np_func, narr):
+    expected = np_func(narr)
+    actual = da_func(darr)
+
+    assert_eq(actual, expected)
+    assert_eq(da_func(narr), expected)  # Ensure Dask reductions work with NumPy arrays
+    assert actual.size == 1
+
+
+def test_reductions_0D():
+    x = np.int_(3)  # np.int_ has a dtype attribute, np.int does not.
+    a = da.from_array(x, chunks=(1,))
+
+    reduction_0d_test(da.sum, a, np.sum, x)
+    reduction_0d_test(da.prod, a, np.prod, x)
+    reduction_0d_test(da.mean, a, np.mean, x)
+    reduction_0d_test(da.var, a, np.var, x)
+    reduction_0d_test(da.std, a, np.std, x)
+    reduction_0d_test(da.min, a, np.min, x)
+    reduction_0d_test(da.max, a, np.max, x)
+    reduction_0d_test(da.any, a, np.any, x)
+    reduction_0d_test(da.all, a, np.all, x)
+
+    reduction_0d_test(da.nansum, a, np.nansum, x)
+    reduction_0d_test(da.nanprod, a, np.nanprod, x)
+    reduction_0d_test(da.nanmean, a, np.mean, x)
+    reduction_0d_test(da.nanvar, a, np.var, x)
+    reduction_0d_test(da.nanstd, a, np.std, x)
+    reduction_0d_test(da.nanmin, a, np.nanmin, x)
+    reduction_0d_test(da.nanmax, a, np.nanmax, x)
+
+
 def reduction_1d_test(da_func, darr, np_func, narr, use_dtype=True, split_every=True):
     assert_eq(da_func(darr), np_func(narr))
     assert_eq(
@@ -97,56 +125,54 @@ def test_reductions_1D(dtype):
 
 
 def reduction_2d_test(da_func, darr, np_func, narr, use_dtype=True, split_every=True):
-    with warnings.catch_warnings():
-        warnings.simplefilter("ignore")  # overflow
-        assert_eq(da_func(darr), np_func(narr))
-        assert_eq(da_func(darr, keepdims=True), np_func(narr, keepdims=True))
-        assert_eq(da_func(darr, axis=()), np_func(narr, axis=()))
-        assert_eq(da_func(darr, axis=0), np_func(narr, axis=0))
-        assert_eq(da_func(darr, axis=1), np_func(narr, axis=1))
-        assert_eq(da_func(darr, axis=-1), np_func(narr, axis=-1))
-        assert_eq(da_func(darr, axis=-2), np_func(narr, axis=-2))
+    assert_eq(da_func(darr), np_func(narr))
+    assert_eq(da_func(darr, keepdims=True), np_func(narr, keepdims=True))
+    assert_eq(da_func(darr, axis=()), np_func(narr, axis=()))
+    assert_eq(da_func(darr, axis=0), np_func(narr, axis=0))
+    assert_eq(da_func(darr, axis=1), np_func(narr, axis=1))
+    assert_eq(da_func(darr, axis=-1), np_func(narr, axis=-1))
+    assert_eq(da_func(darr, axis=-2), np_func(narr, axis=-2))
+    assert_eq(
+        da_func(darr, axis=1, keepdims=True), np_func(narr, axis=1, keepdims=True)
+    )
+    assert_eq(
+        da_func(darr, axis=(), keepdims=True), np_func(narr, axis=(), keepdims=True)
+    )
+    assert_eq(da_func(darr, axis=(1, 0)), np_func(narr, axis=(1, 0)))
+
+    assert same_keys(da_func(darr, axis=()), da_func(darr, axis=()))
+    assert same_keys(da_func(darr, axis=1), da_func(darr, axis=1))
+    assert same_keys(da_func(darr, axis=(1, 0)), da_func(darr, axis=(1, 0)))
+
+    if use_dtype:
+        assert_eq(da_func(darr, dtype="f8"), np_func(narr, dtype="f8"))
+        assert_eq(da_func(darr, dtype="i8"), np_func(narr, dtype="i8"))
+
+    if split_every:
+        a1 = da_func(darr, split_every=4)
+        a2 = da_func(darr, split_every={0: 2, 1: 2})
+        assert same_keys(a1, a2)
+        assert_eq(a1, np_func(narr))
+        assert_eq(a2, np_func(narr))
         assert_eq(
-            da_func(darr, axis=1, keepdims=True), np_func(narr, axis=1, keepdims=True)
+            da_func(darr, keepdims=True, split_every=4),
+            np_func(narr, keepdims=True),
+        )
+        assert_eq(da_func(darr, axis=(), split_every=2), np_func(narr, axis=()))
+        assert_eq(da_func(darr, axis=0, split_every=2), np_func(narr, axis=0))
+        assert_eq(
+            da_func(darr, axis=(), keepdims=True, split_every=2),
+            np_func(narr, axis=(), keepdims=True),
         )
         assert_eq(
-            da_func(darr, axis=(), keepdims=True), np_func(narr, axis=(), keepdims=True)
+            da_func(darr, axis=0, keepdims=True, split_every=2),
+            np_func(narr, axis=0, keepdims=True),
+        )
+        assert_eq(da_func(darr, axis=1, split_every=2), np_func(narr, axis=1))
+        assert_eq(
+            da_func(darr, axis=1, keepdims=True, split_every=2),
+            np_func(narr, axis=1, keepdims=True),
         )
-        assert_eq(da_func(darr, axis=(1, 0)), np_func(narr, axis=(1, 0)))
-
-        assert same_keys(da_func(darr, axis=()), da_func(darr, axis=()))
-        assert same_keys(da_func(darr, axis=1), da_func(darr, axis=1))
-        assert same_keys(da_func(darr, axis=(1, 0)), da_func(darr, axis=(1, 0)))
-
-        if use_dtype:
-            assert_eq(da_func(darr, dtype="f8"), np_func(narr, dtype="f8"))
-            assert_eq(da_func(darr, dtype="i8"), np_func(narr, dtype="i8"))
-
-        if split_every:
-            a1 = da_func(darr, split_every=4)
-            a2 = da_func(darr, split_every={0: 2, 1: 2})
-            assert same_keys(a1, a2)
-            assert_eq(a1, np_func(narr))
-            assert_eq(a2, np_func(narr))
-            assert_eq(
-                da_func(darr, keepdims=True, split_every=4),
-                np_func(narr, keepdims=True),
-            )
-            assert_eq(da_func(darr, axis=(), split_every=2), np_func(narr, axis=()))
-            assert_eq(da_func(darr, axis=0, split_every=2), np_func(narr, axis=0))
-            assert_eq(
-                da_func(darr, axis=(), keepdims=True, split_every=2),
-                np_func(narr, axis=(), keepdims=True),
-            )
-            assert_eq(
-                da_func(darr, axis=0, keepdims=True, split_every=2),
-                np_func(narr, axis=0, keepdims=True),
-            )
-            assert_eq(da_func(darr, axis=1, split_every=2), np_func(narr, axis=1))
-            assert_eq(
-                da_func(darr, axis=1, keepdims=True, split_every=2),
-                np_func(narr, axis=1, keepdims=True),
-            )
 
 
 def test_reduction_errors():
@@ -158,6 +184,7 @@ def test_reduction_errors():
 
 
 @pytest.mark.slow
+@pytest.mark.filterwarnings("ignore:overflow encountered in reduce:RuntimeWarning")
 @pytest.mark.parametrize("dtype", ["f4", "i4"])
 def test_reductions_2D(dtype):
     x = np.arange(1, 122).reshape((11, 11)).astype(dtype)
@@ -228,18 +255,17 @@ def test_nanarg_reductions(dfunc, func):
     a = da.from_array(x, chunks=(3, 4, 5))
     assert_eq(dfunc(a), func(x))
     assert_eq(dfunc(a, 0), func(x, 0))
-    with pytest.raises(ValueError):
-        with pytest.warns(None):  # All NaN axis
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", RuntimeWarning)  # All-NaN slice encountered
+        with pytest.raises(ValueError):
             dfunc(a, 1).compute()
 
-    with pytest.raises(ValueError):
-        with pytest.warns(None):  # All NaN axis
+        with pytest.raises(ValueError):
             dfunc(a, 2).compute()
 
-    x[:] = np.nan
-    a = da.from_array(x, chunks=(3, 4, 5))
-    with pytest.raises(ValueError):
-        with pytest.warns(None):  # All NaN axis
+        x[:] = np.nan
+        a = da.from_array(x, chunks=(3, 4, 5))
+        with pytest.raises(ValueError):
             dfunc(a).compute()
 
 
@@ -294,47 +320,28 @@ def test_reductions_2D_nans():
 
     reduction_2d_test(da.nansum, a, np.nansum, x, False, False)
     reduction_2d_test(da.nanprod, a, np.nanprod, x, False, False)
-    reduction_2d_test(da.nanmean, a, np.nanmean, x, False, False)
-    with pytest.warns(None):  # division by 0 warning
+
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", RuntimeWarning)
+        reduction_2d_test(da.nanmean, a, np.nanmean, x, False, False)
         reduction_2d_test(da.nanvar, a, np.nanvar, x, False, False)
-    with pytest.warns(None):  # division by 0 warning
         reduction_2d_test(da.nanstd, a, np.nanstd, x, False, False)
-    with pytest.warns(None):  # all NaN axis warning
         reduction_2d_test(da.nanmin, a, np.nanmin, x, False, False)
-    with pytest.warns(None):  # all NaN axis warning
         reduction_2d_test(da.nanmax, a, np.nanmax, x, False, False)
 
-    with warnings.catch_warnings():
-        # RuntimeWarning: invalid value encountered in reduce
-        warnings.simplefilter("ignore", RuntimeWarning)
         assert_eq(da.argmax(a), np.argmax(x))
         assert_eq(da.argmin(a), np.argmin(x))
-
-    with pytest.warns(None):  # all NaN axis warning
         assert_eq(da.nanargmax(a), np.nanargmax(x))
-    with pytest.warns(None):  # all NaN axis warning
         assert_eq(da.nanargmin(a), np.nanargmin(x))
 
-    with warnings.catch_warnings():
-        # RuntimeWarning: invalid value encountered in reduce
-        warnings.simplefilter("ignore", RuntimeWarning)
         assert_eq(da.argmax(a, axis=0), np.argmax(x, axis=0))
         assert_eq(da.argmin(a, axis=0), np.argmin(x, axis=0))
-
-    with pytest.warns(None):  # all NaN axis warning
         assert_eq(da.nanargmax(a, axis=0), np.nanargmax(x, axis=0))
-    with pytest.warns(None):  # all NaN axis warning
         assert_eq(da.nanargmin(a, axis=0), np.nanargmin(x, axis=0))
 
-    with warnings.catch_warnings():
-        # RuntimeWarning: invalid value encountered in reduce
-        warnings.simplefilter("ignore", RuntimeWarning)
         assert_eq(da.argmax(a, axis=1), np.argmax(x, axis=1))
         assert_eq(da.argmin(a, axis=1), np.argmin(x, axis=1))
-
-    with pytest.warns(None):  # all NaN axis warning
         assert_eq(da.nanargmax(a, axis=1), np.nanargmax(x, axis=1))
-    with pytest.warns(None):  # all NaN axis warning
         assert_eq(da.nanargmin(a, axis=1), np.nanargmin(x, axis=1))
 
 
@@ -417,7 +424,8 @@ def test_nan_object(func):
             warnings.simplefilter("default", RuntimeWarning)
 
         assert_eq(getattr(np, func)(x, axis=1), getattr(da, func)(d, axis=1))
-        assert_eq(getattr(np, func)(x), getattr(da, func)(d))
+        # wrap the scalar in a numpy array since the dask version cannot know dtype
+        assert_eq(np.array(getattr(np, func)(x)).astype(object), getattr(da, func)(d))
 
 
 def test_0d_array():
@@ -443,7 +451,8 @@ def test_reductions_with_empty_array():
     x2 = dx2.compute()
 
     for dx, x in [(dx1, x1), (dx2, x2)]:
-        with pytest.warns(None):  # empty slice warning
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore", RuntimeWarning)  # Mean of empty slice
             assert_eq(dx.mean(), x.mean())
             assert_eq(dx.mean(axis=()), x.mean(axis=()))
             assert_eq(dx.mean(axis=0), x.mean(axis=0))
@@ -520,11 +529,10 @@ def test_general_reduction_names():
         da.ones(10, dtype, chunks=2), np.sum, np.sum, dtype=dtype, name="foo"
     )
     names, tokens = list(zip_longest(*[key[0].rsplit("-", 1) for key in a.dask]))
-    assert set(names) == {"ones", "foo", "foo-partial", "foo-aggregate"}
+    assert set(names) == {"ones_like", "foo", "foo-partial", "foo-aggregate"}
     assert all(tokens)
 
 
-@pytest.mark.filterwarnings("ignore:`argmax` is not implemented by dask")
 @pytest.mark.parametrize("func", [np.sum, np.argmax])
 def test_array_reduction_out(func):
     x = da.arange(10, chunks=(5,))
@@ -701,6 +709,22 @@ def test_median(axis, keepdims, func):
     )
 
 
+@pytest.mark.parametrize("func", ["median", "nanmedian"])
+@pytest.mark.parametrize("axis", [0, [0, 2], 1])
+def test_median_does_not_rechunk_if_whole_axis_in_one_chunk(axis, func):
+    x = np.arange(100).reshape((2, 5, 10))
+    d = da.from_array(x, chunks=(2, 1, 10))
+
+    actual = getattr(da, func)(d, axis=axis)
+    expected = getattr(np, func)(x, axis=axis)
+    assert_eq(actual, expected)
+    does_rechunk = "rechunk" in str(dict(actual.__dask_graph__()))
+    if axis == 1:
+        assert does_rechunk
+    else:
+        assert not does_rechunk
+
+
 @pytest.mark.parametrize("method", ["sum", "mean", "prod"])
 def test_object_reduction(method):
     arr = da.ones(1).astype(object)
@@ -708,6 +732,48 @@ def test_object_reduction(method):
     assert result == 1
 
 
+@pytest.mark.parametrize("func", ["nanmin", "nanmax"])
+def test_empty_chunk_nanmin_nanmax(func):
+    # see https://github.com/dask/dask/issues/8352
+    x = np.arange(10).reshape(2, 5)
+    d = da.from_array(x, chunks=2)
+    x = x[x > 4]
+    d = d[d > 4]
+    block_lens = np.array([len(x.compute()) for x in d.blocks])
+    assert 0 in block_lens
+    with pytest.raises(ValueError) as err:
+        getattr(da, func)(d)
+    assert "Arrays chunk sizes are unknown" in str(err)
+    d = d.compute_chunk_sizes()
+    assert_eq(getattr(da, func)(d), getattr(np, func)(x))
+
+
+@pytest.mark.parametrize("func", ["nanmin", "nanmax"])
+def test_empty_chunk_nanmin_nanmax_raise(func):
+    # see https://github.com/dask/dask/issues/8352
+    x = np.arange(10).reshape(2, 5)
+    d = da.from_array(x, chunks=2)
+    d = d[d > 9]
+    x = x[x > 9]
+    d = d.compute_chunk_sizes()
+    with pytest.raises(ValueError) as err_np:
+        getattr(np, func)(x)
+    with pytest.raises(ValueError) as err_da:
+        d = getattr(da, func)(d)
+        d.compute()
+    assert str(err_np.value) == str(err_da.value)
+
+
+def test_mean_func_does_not_warn():
+    # non-regression test for https://github.com/pydata/xarray/issues/5151
+    xr = pytest.importorskip("xarray")
+    a = xr.DataArray(da.from_array(np.full((10, 10), np.nan)))
+
+    with warnings.catch_warnings(record=True) as rec:
+        a.mean().compute()
+    assert not rec  # did not warn
+
+
 @pytest.mark.parametrize("func", ["nanvar", "nanstd"])
 def test_nan_func_does_not_warn(func):
     # non-regression test for #6105
@@ -715,6 +781,64 @@ def test_nan_func_does_not_warn(func):
     x[0] = 1
     x[1] = 2
     d = da.from_array(x, chunks=2)
-    with pytest.warns(None) as rec:
+    with warnings.catch_warnings(record=True) as rec:
         getattr(da, func)(d).compute()
     assert not rec  # did not warn
+
+
+@pytest.mark.parametrize("chunks", list(permutations(((2, 1) * 8, (3,) * 8, (6,) * 4))))
+@pytest.mark.parametrize("split_every", [2, 4])
+@pytest.mark.parametrize(
+    "axes", list(permutations((0, 1, 2), 2)) + list(permutations((0, 1, 2)))
+)
+def test_chunk_structure_independence(axes, split_every, chunks):
+    # Reducing an array should not depend on its chunk-structure!!!
+    # See Issue #8541: https://github.com/dask/dask/issues/8541
+    shape = tuple(np.sum(s) for s in chunks)
+    np_array = np.arange(np.prod(shape)).reshape(*shape)
+    x = da.from_array(np_array, chunks=chunks)
+    reduced_x = da.reduction(
+        x,
+        lambda x, axis, keepdims: x,
+        lambda x, axis, keepdims: x,
+        keepdims=True,
+        axis=axes,
+        split_every=split_every,
+        dtype=x.dtype,
+        meta=x._meta,
+    )
+    assert_eq(reduced_x, np_array, check_chunks=False, check_shape=False)
+
+
+def test_weighted_reduction():
+    # Weighted reduction
+    def w_sum(x, weights=None, dtype=None, computing_meta=False, **kwargs):
+        """`chunk` callable for (weighted) sum"""
+        if computing_meta:
+            return x
+        if weights is not None:
+            x = x * weights
+        return np.sum(x, dtype=dtype, **kwargs)
+
+    # Arrays
+    a = 1 + np.ma.arange(60).reshape(6, 10)
+    a[2, 2] = np.ma.masked
+    dx = da.from_array(a, chunks=(4, 5))
+    # Weights
+    w = np.linspace(1, 2, 6).reshape(6, 1)
+
+    # No weights (i.e. normal sum)
+    x = da.reduction(dx, w_sum, np.sum, dtype=dx.dtype)
+    assert_eq(x, np.sum(a), check_shape=True)
+
+    # Weighted sum
+    x = da.reduction(dx, w_sum, np.sum, dtype="f8", weights=w)
+    assert_eq(x, np.sum(a * w), check_shape=True)
+
+    # Non-broadcastable weights (short axis)
+    with pytest.raises(ValueError):
+        da.reduction(dx, w_sum, np.sum, weights=[1, 2, 3])
+
+    # Non-broadcastable weights (too many dims)
+    with pytest.raises(ValueError):
+        da.reduction(dx, w_sum, np.sum, weights=[[[2]]])
