@@ -1,25 +1,16 @@
-import difflib
+import contextlib
 import functools
 import itertools
 import math
 import numbers
-import os
 import warnings
 
 import numpy as np
-from tlz import frequencies, concat
+from tlz import concat, frequencies
 
-from .core import Array
-from ..highlevelgraph import HighLevelGraph
-from ..utils import has_keyword, ignoring, is_arraylike, is_cupy_type
-
-try:
-    AxisError = np.AxisError
-except AttributeError:
-    try:
-        np.array([0]).sum(axis=5)
-    except Exception as e:
-        AxisError = type(e)
+from dask.array.core import Array
+from dask.highlevelgraph import HighLevelGraph
+from dask.utils import has_keyword, is_arraylike, is_cupy_type
 
 
 def normalize_to_array(x):
@@ -65,13 +56,6 @@ def meta_from_array(x, ndim=None, dtype=None):
     if isinstance(x, type):
         x = x(shape=(0,) * (ndim or 0), dtype=dtype)
 
-    if (
-        not hasattr(x, "shape")
-        or not hasattr(x, "dtype")
-        or not isinstance(x.shape, tuple)
-    ):
-        return x
-
     if isinstance(x, list) or isinstance(x, tuple):
         ndims = [
             0
@@ -84,6 +68,13 @@ def meta_from_array(x, ndim=None, dtype=None):
         a = [a if nd == 0 else meta_from_array(a, nd) for a, nd in zip(x, ndims)]
         return a if isinstance(x, list) else tuple(x)
 
+    if (
+        not hasattr(x, "shape")
+        or not hasattr(x, "dtype")
+        or not isinstance(x.shape, tuple)
+    ):
+        return x
+
     if ndim is None:
         ndim = x.ndim
 
@@ -156,8 +147,10 @@ def compute_meta(func, _dtype, *args, **kwargs):
                 else:
                     return None
             except ValueError as e:
-                # min/max functions have no identity, attempt to use the first meta
-                if "zero-size array to reduction operation" in str(e):
+                # min/max functions have no identity, just use the same input type when there's only one
+                if len(
+                    args_meta
+                ) == 1 and "zero-size array to reduction operation" in str(e):
                     meta = args_meta[0]
                 else:
                     return None
@@ -165,7 +158,7 @@ def compute_meta(func, _dtype, *args, **kwargs):
                 return None
 
         if _dtype and getattr(meta, "dtype", None) != _dtype:
-            with ignoring(AttributeError):
+            with contextlib.suppress(AttributeError):
                 meta = meta.astype(_dtype)
 
         if np.isscalar(meta):
@@ -201,7 +194,7 @@ def _not_empty(x):
 
 
 def _check_dsk(dsk):
-    """ Check that graph is well named and non-overlapping """
+    """Check that graph is well named and non-overlapping"""
     if not isinstance(dsk, HighLevelGraph):
         return
 
@@ -221,19 +214,25 @@ def assert_eq_shape(a, b, check_nan=True):
             assert aa == bb
 
 
-def _check_chunks(x):
-    x = x.persist(scheduler="sync")
+def _check_chunks(x, scheduler=None):
+    x = x.persist(scheduler=scheduler)
     for idx in itertools.product(*(range(len(c)) for c in x.chunks)):
         chunk = x.dask[(x.name,) + idx]
+        if hasattr(chunk, "result"):  # it's a future
+            chunk = chunk.result()
         if not hasattr(chunk, "dtype"):
             chunk = np.array(chunk, dtype="O")
         expected_shape = tuple(c[i] for c, i in zip(x.chunks, idx))
         assert_eq_shape(expected_shape, chunk.shape, check_nan=False)
-        assert chunk.dtype == x.dtype
+        assert (
+            chunk.dtype == x.dtype
+        ), "maybe you forgot to pass the scheduler to `assert_eq`?"
     return x
 
 
-def _get_dt_meta_computed(x, check_shape=True, check_graph=True, check_chunks=True):
+def _get_dt_meta_computed(
+    x, check_shape=True, check_graph=True, check_chunks=True, scheduler=None
+):
     x_original = x
     x_meta = None
     x_computed = None
@@ -246,8 +245,8 @@ def _get_dt_meta_computed(x, check_shape=True, check_graph=True, check_chunks=Tr
         x_meta = getattr(x, "_meta", None)
         if check_chunks:
             # Replace x with persisted version to avoid computing it twice.
-            x = _check_chunks(x)
-        x = x.compute(scheduler="sync")
+            x = _check_chunks(x, scheduler=scheduler)
+        x = x.compute(scheduler=scheduler)
         x_computed = x
         if hasattr(x, "todense"):
             x = x.todense()
@@ -272,31 +271,48 @@ def assert_eq(
     check_graph=True,
     check_meta=True,
     check_chunks=True,
+    check_type=True,
+    check_dtype=True,
+    equal_nan=True,
+    scheduler="sync",
     **kwargs,
 ):
     a_original = a
     b_original = b
 
+    if isinstance(a, (list, int, float)):
+        a = np.array(a)
+    if isinstance(b, (list, int, float)):
+        b = np.array(b)
+
     a, adt, a_meta, a_computed = _get_dt_meta_computed(
-        a, check_shape=check_shape, check_graph=check_graph, check_chunks=check_chunks
+        a,
+        check_shape=check_shape,
+        check_graph=check_graph,
+        check_chunks=check_chunks,
+        scheduler=scheduler,
     )
     b, bdt, b_meta, b_computed = _get_dt_meta_computed(
-        b, check_shape=check_shape, check_graph=check_graph, check_chunks=check_chunks
+        b,
+        check_shape=check_shape,
+        check_graph=check_graph,
+        check_chunks=check_chunks,
+        scheduler=scheduler,
     )
 
-    if str(adt) != str(bdt):
-        # Ignore check for matching length of flexible dtypes, since Array._meta
-        # can't encode that information
-        if adt.type == bdt.type and not (adt.type == np.bytes_ or adt.type == np.str_):
-            diff = difflib.ndiff(str(adt).splitlines(), str(bdt).splitlines())
-            raise AssertionError(
-                "string repr are different" + os.linesep + os.linesep.join(diff)
-            )
+    if check_dtype and str(adt) != str(bdt):
+        raise AssertionError(f"a and b have different dtypes: (a: {adt}, b: {bdt})")
 
     try:
         assert (
             a.shape == b.shape
         ), f"a and b have different shapes (a: {a.shape}, b: {b.shape})"
+        if check_type:
+            _a = a if a.shape else a.item()
+            _b = b if b.shape else b.item()
+            assert type(_a) == type(
+                _b
+            ), f"a and b have different types (a: {type(_a)}, b: {type(_b)})"
         if check_meta:
             if hasattr(a, "_meta") and hasattr(b, "_meta"):
                 assert_eq(a._meta, b._meta)
@@ -337,7 +353,7 @@ def assert_eq(
                         )
                         assert type(b_meta) == type(b_computed), msg
         msg = "found values in 'a' and 'b' which differ by more than the allowed amount"
-        assert allclose(a, b, **kwargs), msg
+        assert allclose(a, b, equal_nan=equal_nan, **kwargs), msg
         return True
     except TypeError:
         pass
@@ -373,57 +389,19 @@ def _dtype_of(a):
         return np.asanyarray(a).dtype
 
 
-def empty_like_safe(a, shape, **kwargs):
+def arange_safe(*args, like, **kwargs):
     """
-    Return np.empty_like(a, shape=shape, **kwargs) if the shape argument
-    is supported (requires NumPy >= 1.17), otherwise falls back to
-    using the old behavior, returning np.empty(shape, **kwargs).
+    Use the `like=` from `np.arange` to create a new array dispatching
+    to the downstream library. If that fails, falls back to the
+    default NumPy behavior, resulting in a `numpy.ndarray`.
     """
-    try:
-        return np.empty_like(a, shape=shape, **kwargs)
-    except TypeError:
-        kwargs.setdefault("dtype", _dtype_of(a))
-        return np.empty(shape, **kwargs)
-
-
-def full_like_safe(a, fill_value, shape, **kwargs):
-    """
-    Return np.full_like(a, fill_value, shape=shape, **kwargs) if the
-    shape argument is supported (requires NumPy >= 1.17), otherwise
-    falls back to using the old behavior, returning
-    np.full(shape, fill_value, **kwargs).
-    """
-    try:
-        return np.full_like(a, fill_value, shape=shape, **kwargs)
-    except TypeError:
-        kwargs.setdefault("dtype", _dtype_of(a))
-        return np.full(shape, fill_value, **kwargs)
-
-
-def ones_like_safe(a, shape, **kwargs):
-    """
-    Return np.ones_like(a, shape=shape, **kwargs) if the shape argument
-    is supported (requires NumPy >= 1.17), otherwise falls back to
-    using the old behavior, returning np.ones(shape, **kwargs).
-    """
-    try:
-        return np.ones_like(a, shape=shape, **kwargs)
-    except TypeError:
-        kwargs.setdefault("dtype", _dtype_of(a))
-        return np.ones(shape, **kwargs)
-
-
-def zeros_like_safe(a, shape, **kwargs):
-    """
-    Return np.zeros_like(a, shape=shape, **kwargs) if the shape argument
-    is supported (requires NumPy >= 1.17), otherwise falls back to
-    using the old behavior, returning np.zeros(shape, **kwargs).
-    """
-    try:
-        return np.zeros_like(a, shape=shape, **kwargs)
-    except TypeError:
-        kwargs.setdefault("dtype", _dtype_of(a))
-        return np.zeros(shape, **kwargs)
+    if like is None:
+        return np.arange(*args, **kwargs)
+    else:
+        try:
+            return np.arange(*args, like=meta_from_array(like), **kwargs)
+        except TypeError:
+            return np.arange(*args, **kwargs)
 
 
 def _array_like_safe(np_func, da_func, a, like, **kwargs):
@@ -453,7 +431,7 @@ def array_safe(a, like, **kwargs):
     to convert a `dask.Array` and CuPy doesn't implement `__array__` to
     prevent implicit copies to host.
     """
-    from .routines import array
+    from dask.array.routines import array
 
     return _array_like_safe(np.array, array, a, like, **kwargs)
 
@@ -467,7 +445,7 @@ def asarray_safe(a, like, **kwargs):
     a.compute(scheduler="sync") before np.asarray, as downstream
     libraries are unlikely to know how to convert a dask.Array.
     """
-    from .core import asarray
+    from dask.array.core import asarray
 
     return _array_like_safe(np.asarray, asarray, a, like, **kwargs)
 
@@ -481,19 +459,19 @@ def asanyarray_safe(a, like, **kwargs):
     a.compute(scheduler="sync") before np.asanyarray, as downstream
     libraries are unlikely to know how to convert a dask.Array.
     """
-    from .core import asanyarray
+    from dask.array.core import asanyarray
 
     return _array_like_safe(np.asanyarray, asanyarray, a, like, **kwargs)
 
 
 def validate_axis(axis, ndim):
-    """ Validate an input to axis= keywords """
+    """Validate an input to axis= keywords"""
     if isinstance(axis, (tuple, list)):
         return tuple(validate_axis(ax, ndim) for ax in axis)
     if not isinstance(axis, numbers.Integral):
         raise TypeError("Axis value must be an integer, got %s" % axis)
     if axis < -ndim or axis >= ndim:
-        raise AxisError(
+        raise np.AxisError(
             "Axis %d is out of bounds for array of dimension %d" % (axis, ndim)
         )
     if axis < 0:
@@ -543,15 +521,35 @@ def svd_flip(u, v, u_based_decision=False):
     return u, v
 
 
-def _is_nep18_active():
-    class A:
-        def __array_function__(self, *args, **kwargs):
-            return True
+def scipy_linalg_safe(func_name, *args, **kwargs):
+    # need to evaluate at least the first input array
+    # for gpu/cpu checking
+    a = args[0]
+    if is_cupy_type(a):
+        import cupyx.scipy.linalg
 
-    try:
-        return np.concatenate([A()])
-    except ValueError:
-        return False
+        func = getattr(cupyx.scipy.linalg, func_name)
+    else:
+        import scipy.linalg
+
+        func = getattr(scipy.linalg, func_name)
+
+    return func(*args, **kwargs)
 
 
-IS_NEP18_ACTIVE = _is_nep18_active()
+def solve_triangular_safe(a, b, lower=False):
+    return scipy_linalg_safe("solve_triangular", a, b, lower=lower)
+
+
+def __getattr__(name):
+    # Can't use the @_deprecated decorator as it would not work on `except AxisError`
+    if name == "AxisError":
+        warnings.warn(
+            "AxisError was deprecated after version 2021.10.0 and will be removed in a "
+            "future release. Please use numpy.AxisError instead.",
+            category=FutureWarning,
+            stacklevel=2,
+        )
+        return np.AxisError
+    else:
+        raise AttributeError(f"module {__name__} has no attribute {name}")
