@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import os
 import subprocess
 import sys
@@ -13,7 +15,11 @@ from dask.utils import tmpdir
 
 files = ["a", "b"]
 requests = pytest.importorskip("requests")
-errs = requests.exceptions.RequestException, FileNotFoundError
+
+errs: tuple[type[Exception], ...] = (
+    requests.exceptions.RequestException,
+    FileNotFoundError,
+)
 if parse_version(fsspec.__version__) > parse_version("0.7.4"):
     aiohttp = pytest.importorskip("aiohttp")
     errs = errs + (aiohttp.client_exceptions.ClientResponseError,)
@@ -115,8 +121,12 @@ def test_ops_blocksize(dir_server):
         fn = files[1]
         f = open_files(root + fn, block_size=2)[0]
         with f as f:
-            # fails because we want only 12 bytes
-            with pytest.raises(ValueError):
+            if parse_version(fsspec.__version__) < parse_version("2021.11.1"):
+                # fails because we want only 12 bytes
+                with pytest.raises(ValueError):
+                    assert f.read(10) == expected[:10]
+            else:
+                # fixed in https://github.com/fsspec/filesystem_spec/pull/830
                 assert f.read(10) == expected[:10]
 
 
