@@ -1,9 +1,10 @@
+import os
 from collections.abc import Mapping
 from io import BytesIO
 from warnings import catch_warnings, simplefilter, warn
 
-from ...highlevelgraph import HighLevelGraph
-from ...layers import DataFrameIOLayer
+from dask.highlevelgraph import HighLevelGraph
+from dask.layers import DataFrameIOLayer
 
 try:
     import psutil
@@ -26,13 +27,13 @@ from pandas.api.types import (
     is_object_dtype,
 )
 
-from ...base import tokenize
-from ...bytes import read_bytes
-from ...core import flatten
-from ...delayed import delayed
-from ...utils import asciitable, parse_bytes
-from ..core import new_dd_object
-from ..utils import clear_known_categories
+from dask.base import tokenize
+from dask.bytes import read_bytes
+from dask.core import flatten
+from dask.dataframe.core import new_dd_object
+from dask.dataframe.utils import clear_known_categories
+from dask.delayed import delayed
+from dask.utils import asciitable, parse_bytes
 
 
 class CSVFunctionWrapper:
@@ -105,6 +106,8 @@ class CSVFunctionWrapper:
         if not is_first:
             write_header = True
             rest_kwargs.pop("skiprows", None)
+            if rest_kwargs.get("header", 0) is not None:
+                rest_kwargs.pop("header", None)
         if not is_last:
             rest_kwargs.pop("skipfooter", None)
 
@@ -434,7 +437,7 @@ def auto_blocksize(total_memory, cpu_count):
 
 
 def _infer_block_size():
-    default = 2 ** 25
+    default = 2**25
     if psutil is not None:
         with catch_warnings():
             simplefilter("ignore", RuntimeWarning)
@@ -570,7 +573,23 @@ def read_pandas(
     names = kwargs.get("names", None)
     header = kwargs.get("header", "infer" if names is None else None)
     need = 1 if header is None else 2
-    parts = b_sample.split(b_lineterminator, lastskiprow + need)
+
+    if kwargs.get("comment"):
+        # if comment is provided, step through lines of b_sample and strip out comments
+        parts = []
+        for part in b_sample.split(b_lineterminator):
+            split_comment = part.decode().split(kwargs.get("comment"))
+            if len(split_comment) > 1:
+                # if line starts with comment, don't include that line in parts.
+                if len(split_comment[0]) > 0:
+                    parts.append(split_comment[0].strip().encode())
+            else:
+                parts.append(part)
+            if len(parts) > need:
+                break
+    else:
+        parts = b_sample.split(b_lineterminator, lastskiprow + need)
+
     # If the last partition is empty, don't count it
     nparts = 0 if not parts else len(parts) - int(not parts[-1])
 
@@ -581,6 +600,8 @@ def read_pandas(
             "in `sample` in the call to `read_csv`/`read_table`"
         )
 
+    if isinstance(header, int):
+        firstrow += header
     header = b"" if header is None else parts[firstrow] + b_lineterminator
 
     # Use sample to infer dtypes and check for presence of include_path_column
@@ -746,7 +767,7 @@ read_fwf = make_reader(pd.read_fwf, "read_fwf", "fixed-width")
 def _write_csv(df, fil, *, depend_on=None, **kwargs):
     with fil as f:
         df.to_csv(f, **kwargs)
-    return None
+    return os.path.normpath(fil.path)
 
 
 def to_csv(
@@ -933,12 +954,11 @@ def to_csv(
 
         import dask
 
-        dask.compute(*values, **compute_kwargs)
-        return [f.path for f in files]
+        return list(dask.compute(*values, **compute_kwargs))
     else:
         return values
 
 
-from ..core import _Frame
+from dask.dataframe.core import _Frame
 
 _Frame.to_csv.__doc__ = to_csv.__doc__
