@@ -1,28 +1,37 @@
+import os
 from math import ceil
 from operator import getitem
-import os
 from threading import Lock
 
-import pandas as pd
 import numpy as np
+import pandas as pd
 from tlz import merge
 
-from ...base import tokenize
-from ... import array as da
-from ...dataframe.core import new_dd_object
-from ...delayed import delayed
-
-from ..core import DataFrame, Series, Index, new_dd_object, has_parallel_type
-from ..shuffle import set_partition
-from ..utils import insert_meta_param_description, check_meta, make_meta, is_series_like
-
-from ...utils import M, ensure_dict
+import dask.array as da
+from dask.base import tokenize
+from dask.dataframe.core import (
+    DataFrame,
+    Index,
+    Series,
+    has_parallel_type,
+    new_dd_object,
+)
+from dask.dataframe.shuffle import set_partition
+from dask.dataframe.utils import (
+    check_meta,
+    insert_meta_param_description,
+    is_series_like,
+    make_meta,
+)
+from dask.delayed import delayed
+from dask.highlevelgraph import HighLevelGraph
+from dask.utils import M, _deprecated, ensure_dict
 
 lock = Lock()
 
 
 def _meta_from_array(x, columns=None, index=None, meta=None):
-    """ Create empty DataFrame or Series which has correct dtype """
+    """Create empty DataFrame or Series which has correct dtype"""
 
     if x.ndim > 2:
         raise ValueError(
@@ -46,7 +55,7 @@ def _meta_from_array(x, columns=None, index=None, meta=None):
             raise ValueError("For a struct dtype, columns must be a list.")
         elif not all(i in x.dtype.names for i in columns):
             extra = sorted(set(columns).difference(x.dtype.names))
-            raise ValueError("dtype {0} doesn't have fields {1}".format(x.dtype, extra))
+            raise ValueError(f"dtype {x.dtype} doesn't have fields {extra}")
         fields = x.dtype.fields
         dtypes = [fields[n][0] if n in fields else "f8" for n in columns]
     elif x.ndim == 1:
@@ -68,9 +77,8 @@ def _meta_from_array(x, columns=None, index=None, meta=None):
             columns = list(range(x.shape[1])) if x.ndim == 2 else [0]
         elif len(columns) != x.shape[1]:
             raise ValueError(
-                "Number of column names must match width of the "
-                "array. Got {0} names for {1} "
-                "columns".format(len(columns), x.shape[1])
+                "Number of column names must match width of the array. "
+                f"Got {len(columns)} names for {x.shape[1]} columns"
             )
         dtypes = [x.dtype] * len(columns)
 
@@ -137,7 +145,11 @@ def from_pandas(data, npartitions=None, chunksize=None, sort=True, name=None):
 
     This splits an in-memory Pandas dataframe into several parts and constructs
     a dask.dataframe from those parts on which Dask.dataframe can operate in
-    parallel.
+    parallel.  By default, the input dataframe will be sorted by the index to
+    produce cleanly-divided partitions (with known divisions).  To preserve the
+    input ordering, make sure the input index is monotonically-increasing. The
+    ``sort=False`` option will also avoid reordering, but will not result in
+    known divisions.
 
     Note that, despite parallelism, Dask.dataframe may not always be faster
     than Pandas.  We recommend that you stay with Pandas for as long as
@@ -154,8 +166,9 @@ def from_pandas(data, npartitions=None, chunksize=None, sort=True, name=None):
     chunksize : int, optional
         The number of rows per index partition to use.
     sort: bool
-        Sort input first to obtain cleanly divided partitions or don't sort and
-        don't get cleanly divided partitions
+        Sort the input by index first to obtain cleanly divided partitions
+        (with known divisions).  If False, the input will not be sorted, and
+        all divisions will be set to None. Default is True.
     name: string, optional
         An optional keyname for the dataframe.  Defaults to hashing the input
 
@@ -229,6 +242,7 @@ def from_pandas(data, npartitions=None, chunksize=None, sort=True, name=None):
     return new_dd_object(dsk, name, data, divisions)
 
 
+@_deprecated(after_version="2022.02.1")
 def from_bcolz(x, chunksize=None, categorize=True, index=None, lock=lock, **kwargs):
     """Read BColz CTable into a Dask Dataframe
 
@@ -254,9 +268,10 @@ def from_bcolz(x, chunksize=None, categorize=True, index=None, lock=lock, **kwar
     if lock is True:
         lock = Lock()
 
-    import dask.array as da
     import bcolz
 
+    import dask.array as da
+
     if isinstance(x, str):
         x = bcolz.ctable(rootdir=x)
     bc_chunklen = max(x[name].chunklen for name in x.names)
@@ -291,20 +306,17 @@ def from_bcolz(x, chunksize=None, categorize=True, index=None, lock=lock, **kwar
         )
     new_name = "from_bcolz-" + token
 
-    dsk = dict(
-        (
-            (new_name, i),
-            (
-                dataframe_from_ctable,
-                x,
-                (slice(i * chunksize, (i + 1) * chunksize),),
-                columns,
-                categories,
-                lock,
-            ),
+    dsk = {
+        (new_name, i): (
+            dataframe_from_ctable,
+            x,
+            (slice(i * chunksize, (i + 1) * chunksize),),
+            columns,
+            categories,
+            lock,
         )
         for i in range(0, int(ceil(len(x) / chunksize)))
-    )
+    }
 
     meta = dataframe_from_ctable(x, slice(0, 0), columns, categories, lock)
     result = DataFrame(dsk, new_name, meta, divisions)
@@ -495,14 +507,26 @@ def _link(token, result):
     return None
 
 
-def _df_to_bag(df, index=False):
+def _df_to_bag(df, index=False, format="tuple"):
     if isinstance(df, pd.DataFrame):
-        return list(map(tuple, df.itertuples(index)))
+        if format == "tuple":
+            return list(map(tuple, df.itertuples(index)))
+        elif format == "dict":
+            if index:
+                return [
+                    {**{"index": idx}, **values}
+                    for values, idx in zip(df.to_dict("records"), df.index)
+                ]
+            else:
+                return df.to_dict(orient="records")
     elif isinstance(df, pd.Series):
-        return list(df.iteritems()) if index else list(df)
+        if format == "tuple":
+            return list(df.items()) if index else list(df)
+        elif format == "dict":
+            return df.to_frame().to_dict(orient="records")
 
 
-def to_bag(df, index=False):
+def to_bag(df, index=False, format="tuple"):
     """Create Dask Bag from a Dask DataFrame
 
     Parameters
@@ -510,21 +534,33 @@ def to_bag(df, index=False):
     index : bool, optional
         If True, the elements are tuples of ``(index, value)``, otherwise
         they're just the ``value``.  Default is False.
+    format : {"tuple", "dict", "frame"}, optional
+        Whether to return a bag of tuples, dictionaries, or
+        dataframe-like objects. Default is "tuple". If "frame",
+        the original partitions of ``df`` will not be transformed
+        in any way.
+
 
     Examples
     --------
     >>> bag = df.to_bag()  # doctest: +SKIP
     """
-    from ...bag.core import Bag
+    from dask.bag.core import Bag
 
     if not isinstance(df, (DataFrame, Series)):
         raise TypeError("df must be either DataFrame or Series")
-    name = "to_bag-" + tokenize(df, index)
-    dsk = dict(
-        ((name, i), (_df_to_bag, block, index))
-        for (i, block) in enumerate(df.__dask_keys__())
-    )
-    dsk.update(df.__dask_optimize__(df.__dask_graph__(), df.__dask_keys__()))
+    name = "to_bag-" + tokenize(df, index, format)
+    if format == "frame":
+        # Use existing graph and name of df, but
+        # drop meta to produce a Bag collection
+        dsk = df.dask
+        name = df._name
+    else:
+        dsk = {
+            (name, i): (_df_to_bag, block, index, format)
+            for (i, block) in enumerate(df.__dask_keys__())
+        }
+        dsk.update(df.__dask_optimize__(df.__dask_graph__(), df.__dask_keys__()))
     return Bag(dsk, name, df.npartitions)
 
 
@@ -538,7 +574,6 @@ def to_records(df):
     Examples
     --------
     >>> df.to_records()  # doctest: +SKIP
-    dask.array<to_records, shape=(nan,), dtype=(numpy.record, [('ind', '<f8'), ('x', 'O'), ('y', '<i8')]), chunksize=(nan,), chunktype=numpy.ndarray>  # noqa: E501
 
     See Also
     --------
@@ -556,10 +591,11 @@ def from_delayed(
 
     Parameters
     ----------
-    dfs : list of Delayed
+    dfs : list of Delayed or Future
         An iterable of ``dask.delayed.Delayed`` objects, such as come from
-        ``dask.delayed`` These comprise the individual partitions of the
-        resulting dataframe.
+        ``dask.delayed`` or an iterable of ``distributed.Future`` objects,
+        such as come from ``client.submit`` interface. These comprise the individual
+        partitions of the resulting dataframe.
     $META
     divisions : tuple, str, optional
         Partition boundaries along the index.
@@ -580,6 +616,7 @@ def from_delayed(
         delayed(df) if not isinstance(df, Delayed) and hasattr(df, "key") else df
         for df in dfs
     ]
+
     for df in dfs:
         if not isinstance(df, Delayed):
             raise TypeError("Expected Delayed object, got %s" % type(df).__name__)
@@ -589,8 +626,11 @@ def from_delayed(
     else:
         meta = make_meta(meta)
 
+    if not dfs:
+        dfs = [delayed(make_meta)(meta)]
+
     name = prefix + "-" + tokenize(*dfs)
-    dsk = merge(df.dask for df in dfs)
+    dsk = {}
     if verify_meta:
         for (i, df) in enumerate(dfs):
             dsk[(name, i)] = (check_meta, df.key, meta, "from_delayed")
@@ -605,10 +645,12 @@ def from_delayed(
         if len(divs) != len(dfs) + 1:
             raise ValueError("divisions should be a tuple of len(dfs) + 1")
 
-    df = new_dd_object(dsk, name, meta, divs)
+    df = new_dd_object(
+        HighLevelGraph.from_collections(name, dsk, dfs), name, meta, divs
+    )
 
     if divisions == "sorted":
-        from ..shuffle import compute_and_set_divisions
+        from dask.dataframe.shuffle import compute_and_set_divisions
 
         df = compute_and_set_divisions(df)
 
