@@ -2,7 +2,6 @@ import json
 import warnings
 from collections import defaultdict
 from datetime import datetime
-from functools import partial
 
 import numpy as np
 import pandas as pd
@@ -10,25 +9,30 @@ import pyarrow as pa
 import pyarrow.parquet as pq
 from packaging.version import parse as parse_version
 
-from dask import delayed
-
-from ....base import tokenize
-from ....core import flatten
-from ....delayed import Delayed
-from ....utils import getargspec, natural_sort_key
-from ...utils import clear_known_categories
-from ..utils import _get_pyarrow_dtypes, _meta_from_dtypes
-from .core import create_metadata_file
-from .utils import (
+from dask.base import tokenize
+from dask.core import flatten
+from dask.dataframe.io.parquet.core import create_metadata_file
+from dask.dataframe.io.parquet.utils import (
     Engine,
     _flatten_filters,
     _get_aggregation_depth,
     _normalize_index_columns,
     _parse_pandas_metadata,
+    _process_open_file_options,
     _row_groups_to_parts,
     _set_metadata_task_size,
     _sort_and_analyze_paths,
+    _split_user_options,
 )
+from dask.dataframe.io.utils import (
+    _get_pyarrow_dtypes,
+    _is_local_fs,
+    _meta_from_dtypes,
+    _open_input_files,
+)
+from dask.dataframe.utils import clear_known_categories
+from dask.delayed import Delayed, delayed
+from dask.utils import getargspec, natural_sort_key
 
 # Check PyArrow version for feature support
 _pa_version = parse_version(pa.__version__)
@@ -63,7 +67,16 @@ def _append_row_groups(metadata, md):
 
 
 def _write_partitioned(
-    table, root_path, filename, partition_cols, fs, index_cols=(), **kwargs
+    table,
+    df,
+    root_path,
+    filename,
+    partition_cols,
+    fs,
+    pandas_to_arrow_table,
+    preserve_index,
+    index_cols=(),
+    **kwargs,
 ):
     """Write table to a partitioned dataset with pyarrow.
 
@@ -78,7 +91,10 @@ def _write_partitioned(
     """
     fs.mkdirs(root_path, exist_ok=True)
 
-    df = table.to_pandas(ignore_metadata=True)
+    if preserve_index:
+        df.reset_index(inplace=True)
+    df = df[table.schema.names]
+
     index_cols = list(index_cols) if index_cols else []
     preserve_index = False
     if index_cols:
@@ -103,12 +119,8 @@ def _write_partitioned(
         subdir = fs.sep.join(
             [f"{name}={val}" for name, val in zip(partition_cols, keys)]
         )
-        subtable = pa.Table.from_pandas(
-            subgroup,
-            nthreads=1,
-            preserve_index=preserve_index,
-            schema=subschema,
-            safe=False,
+        subtable = pandas_to_arrow_table(
+            subgroup, preserve_index=preserve_index, schema=subschema
         )
         prefix = fs.sep.join([root_path, subdir])
         fs.mkdirs(prefix, exist_ok=True)
@@ -197,17 +209,45 @@ def _read_table_from_path(
     are specified (otherwise fragments are converted directly
     into tables).
     """
+
+    # Define file-opening options
+    read_kwargs = kwargs.get("read", {}).copy()
+    precache_options, open_file_options = _process_open_file_options(
+        read_kwargs.pop("open_file_options", {}),
+        **(
+            {
+                "allow_precache": False,
+                "default_cache": "none",
+            }
+            if _is_local_fs(fs)
+            else {
+                "columns": columns,
+                "row_groups": row_groups if row_groups == [None] else [row_groups],
+                "default_engine": "pyarrow",
+                "default_cache": "none",
+            }
+        ),
+    )
+
     if partition_keys:
         tables = []
-        for rg in row_groups:
-            piece = pq.ParquetDatasetPiece(
-                path,
-                row_group=rg,
-                partition_keys=partition_keys,
-                open_file_func=partial(fs.open, mode="rb"),
-            )
-            arrow_table = piece_to_arrow_func(piece, columns, partitions, **kwargs)
-            tables.append(arrow_table)
+        with _open_input_files(
+            [path],
+            fs=fs,
+            precache_options=precache_options,
+            **open_file_options,
+        )[0] as fil:
+            for rg in row_groups:
+                piece = pq.ParquetDatasetPiece(
+                    path,
+                    row_group=rg,
+                    partition_keys=partition_keys,
+                    open_file_func=lambda _path, **_kwargs: fil,
+                )
+                arrow_table = piece_to_arrow_func(
+                    piece, columns, partitions, **read_kwargs
+                )
+                tables.append(arrow_table)
 
         if len(row_groups) > 1:
             # NOTE: Not covered by pytest
@@ -215,12 +255,18 @@ def _read_table_from_path(
         else:
             return tables[0]
     else:
-        with fs.open(path, mode="rb") as fil:
+        with _open_input_files(
+            [path],
+            fs=fs,
+            precache_options=precache_options,
+            **open_file_options,
+        )[0] as fil:
             if row_groups == [None]:
                 return pq.ParquetFile(fil).read(
                     columns=columns,
                     use_threads=False,
                     use_pandas_metadata=True,
+                    **read_kwargs,
                 )
             else:
                 return pq.ParquetFile(fil).read_row_groups(
@@ -228,6 +274,7 @@ def _read_table_from_path(
                     columns=columns,
                     use_threads=False,
                     use_pandas_metadata=True,
+                    **read_kwargs,
                 )
 
 
@@ -328,7 +375,7 @@ class ArrowDatasetEngine(Engine):
             aggregate_files,
             ignore_metadata_file,
             metadata_task_size,
-            **kwargs.get("dataset", {}),
+            kwargs,
         )
 
         # Stage 2: Generate output `meta`
@@ -598,7 +645,7 @@ class ArrowDatasetEngine(Engine):
                 # TODO Coerce values for compatible but different dtypes
                 raise ValueError(
                     "Appended dtypes differ.\n{}".format(
-                        set(dtypes.items()) ^ set(df.dtypes.iteritems())
+                        set(dtypes.items()) ^ set(df.dtypes.items())
                     )
                 )
 
@@ -673,10 +720,13 @@ class ArrowDatasetEngine(Engine):
         if partition_on:
             md_list = _write_partitioned(
                 t,
+                df,
                 path,
                 filename,
                 partition_on,
                 fs,
+                cls._pandas_to_arrow_table,
+                preserve_index,
                 index_cols=index_cols,
                 compression=compression,
                 **kwargs,
@@ -708,8 +758,8 @@ class ArrowDatasetEngine(Engine):
         else:
             return []
 
-    @staticmethod
-    def write_metadata(parts, fmd, fs, path, append=False, **kwargs):
+    @classmethod
+    def write_metadata(cls, parts, meta, fs, path, append=False, **kwargs):
         schema = parts[0][0].get("schema", None)
         parts = [p for p in parts if p[0]["meta"] is not None]
         if parts:
@@ -723,8 +773,8 @@ class ArrowDatasetEngine(Engine):
 
             # Aggregate metadata and write to _metadata file
             metadata_path = fs.sep.join([path, "_metadata"])
-            if append and fmd is not None:
-                _meta = fmd
+            if append and meta is not None:
+                _meta = meta
                 i_start = 0
             else:
                 _meta = parts[0][0]["meta"]
@@ -752,7 +802,7 @@ class ArrowDatasetEngine(Engine):
         aggregate_files,
         ignore_metadata_file,
         metadata_task_size,
-        **dataset_kwargs,
+        kwargs,
     ):
         """pyarrow.dataset version of _collect_dataset_info
         Use pyarrow.dataset API to construct a dictionary of all
@@ -764,7 +814,13 @@ class ArrowDatasetEngine(Engine):
         # Use pyarrow.dataset API
         ds = None
         valid_paths = None  # Only used if `paths` is a list containing _metadata
-        _dataset_kwargs = dataset_kwargs.copy()
+
+        # Extract "supported" key-word arguments from `kwargs`
+        (
+            _dataset_kwargs,
+            read_kwargs,
+            user_kwargs,
+        ) = _split_user_options(**kwargs)
 
         # Discover Partitioning - Note that we need to avoid creating
         # this factory until it is actually used.  The `partitioning`
@@ -777,9 +833,10 @@ class ArrowDatasetEngine(Engine):
             {"obj": pa_ds.HivePartitioning},
         )
 
-        # Check that we are not silently ignoring any dataset_kwargs
-        if _dataset_kwargs:
-            raise ValueError(f"Unsupported dataset_kwargs: {_dataset_kwargs.keys()}")
+        # Set require_extension option
+        require_extension = _dataset_kwargs.pop(
+            "require_extension", (".parq", ".parquet")
+        )
 
         # Case-dependent pyarrow.dataset creation
         has_metadata_file = False
@@ -800,10 +857,23 @@ class ArrowDatasetEngine(Engine):
                         *partitioning.get("args", []),
                         **partitioning.get("kwargs", {}),
                     ),
+                    **_dataset_kwargs,
                 )
                 has_metadata_file = True
                 if gather_statistics is None:
                     gather_statistics = True
+            elif require_extension:
+                # Need to materialize all paths if we are missing the _metadata file
+                # Raise error if all files have been filtered by extension
+                len0 = len(paths)
+                paths = [
+                    path for path in fs.find(paths) if path.endswith(require_extension)
+                ]
+                if len0 and paths == []:
+                    raise ValueError(
+                        "No files satisfy the `require_extension` criteria "
+                        f"(files must end with {require_extension})."
+                    )
 
         elif len(paths) > 1:
             paths, base, fns = _sort_and_analyze_paths(paths, fs)
@@ -819,6 +889,7 @@ class ArrowDatasetEngine(Engine):
                             *partitioning.get("args", []),
                             **partitioning.get("kwargs", {}),
                         ),
+                        **_dataset_kwargs,
                     )
                     has_metadata_file = True
                     if gather_statistics is None:
@@ -839,6 +910,7 @@ class ArrowDatasetEngine(Engine):
                     *partitioning.get("args", []),
                     **partitioning.get("kwargs", {}),
                 ),
+                **_dataset_kwargs,
             )
 
         # At this point, we know if `split_row_groups` should be
@@ -951,6 +1023,11 @@ class ArrowDatasetEngine(Engine):
             "partition_names": partition_names,
             "partitioning": partitioning,
             "metadata_task_size": metadata_task_size,
+            "kwargs": {
+                "dataset": _dataset_kwargs,
+                "read": read_kwargs,
+                **user_kwargs,
+            },
         }
 
     @classmethod
@@ -967,6 +1044,7 @@ class ArrowDatasetEngine(Engine):
         categories = dataset_info["categories"]
         partition_obj = dataset_info["partitions"]
         partitions = dataset_info["partition_names"]
+        physical_column_names = dataset_info.get("physical_schema", schema).names
         columns = None
 
         # Set index and column names using
@@ -989,7 +1067,7 @@ class ArrowDatasetEngine(Engine):
         else:
             # No pandas metadata implies no index, unless selected by the user
             index_names = []
-            column_names = dataset_info.get("physical_schema", schema).names
+            column_names = physical_column_names
             storage_name_mapping = {k: k for k in column_names}
             column_index_names = [None]
         if index is None and index_names:
@@ -999,7 +1077,7 @@ class ArrowDatasetEngine(Engine):
         # Ensure that there is no overlap between partition columns
         # and explicit column storage
         if partitions:
-            _partitions = [p for p in partitions if p not in column_names]
+            _partitions = [p for p in partitions if p not in physical_column_names]
             if not _partitions:
                 partitions = []
                 dataset_info["partitions"] = None
@@ -1009,7 +1087,9 @@ class ArrowDatasetEngine(Engine):
                 raise ValueError(
                     "No partition-columns should be written in the \n"
                     "file unless they are ALL written in the file.\n"
-                    "columns: {} | partitions: {}".format(column_names, partitions)
+                    "physical columns: {} | partitions: {}".format(
+                        physical_column_names, partitions
+                    )
                 )
 
         column_names, index_names = _normalize_index_columns(
@@ -1101,6 +1181,7 @@ class ArrowDatasetEngine(Engine):
         categories = dataset_info["categories"]
         has_metadata_file = dataset_info["has_metadata_file"]
         valid_paths = dataset_info["valid_paths"]
+        kwargs = dataset_info["kwargs"]
 
         # Ensure metadata_task_size is set
         # (Using config file or defaults)
@@ -1151,6 +1232,7 @@ class ArrowDatasetEngine(Engine):
             "categories": categories,
             "filters": filters,
             "schema": schema,
+            **kwargs,
         }
 
         # Check if this is a very simple case where we can just return
@@ -1486,6 +1568,7 @@ class ArrowDatasetEngine(Engine):
                         *partitioning.get("args", []),
                         **partitioning.get("kwargs", {}),
                     ),
+                    **kwargs.get("dataset", {}),
                 )
                 frags = list(ds.get_fragments())
                 assert len(frags) == 1
@@ -1673,7 +1756,7 @@ class ArrowLegacyEngine(ArrowDatasetEngine):
         aggregate_files,
         ignore_metadata_file,
         metadata_task_size,
-        **dataset_kwargs,
+        kwargs,
     ):
         """pyarrow-legacy version of _collect_dataset_info
         Use the ParquetDataset API to construct a dictionary of all
@@ -1688,6 +1771,13 @@ class ArrowLegacyEngine(ArrowDatasetEngine):
         if metadata_task_size:
             raise ValueError("metadata_task_size not supported in ArrowLegacyEngine")
 
+        # Extract "supported" key-word arguments from `kwargs`
+        (
+            dataset_kwargs,
+            read_kwargs,
+            user_kwargs,
+        ) = _split_user_options(**kwargs)
+
         (
             schema,
             metadata,
@@ -1727,6 +1817,11 @@ class ArrowLegacyEngine(ArrowDatasetEngine):
             "partition_keys": partition_info["partition_keys"],
             "partition_names": partition_info["partition_names"],
             "partitions": partition_info["partitions"],
+            "kwargs": {
+                "dataset": dataset_kwargs,
+                "read": read_kwargs,
+                **user_kwargs,
+            },
         }
 
     @classmethod
@@ -1754,6 +1849,7 @@ class ArrowLegacyEngine(ArrowDatasetEngine):
             dataset_info["gather_statistics"],
             dataset_info["chunksize"],
             dataset_info["aggregation_depth"],
+            dataset_info["kwargs"],
         )
 
     @classmethod
@@ -1929,6 +2025,7 @@ class ArrowLegacyEngine(ArrowDatasetEngine):
         gather_statistics,
         chunksize,
         aggregation_depth,
+        kwargs,
     ):
         """Construct ``parts`` for ddf construction
 
@@ -1954,7 +2051,11 @@ class ArrowLegacyEngine(ArrowDatasetEngine):
             for full_path in metadata:
                 part = {"piece": (full_path, None, partition_keys.get(full_path, None))}
                 parts.append(part)
-            common_kwargs = {"partitions": partition_obj, "categories": categories}
+            common_kwargs = {
+                "partitions": partition_obj,
+                "categories": categories,
+                **kwargs,
+            }
             return parts, stats, common_kwargs
 
         # Use final metadata info to update our options for
@@ -1989,6 +2090,7 @@ class ArrowLegacyEngine(ArrowDatasetEngine):
             fs,
             chunksize,
             aggregation_depth,
+            kwargs,
         )
 
     @classmethod
@@ -2195,6 +2297,7 @@ class ArrowLegacyEngine(ArrowDatasetEngine):
         fs,
         chunksize,
         aggregation_depth,
+        kwargs,
     ):
         """Process row-groups and statistics.
 
@@ -2240,6 +2343,7 @@ class ArrowLegacyEngine(ArrowDatasetEngine):
             "partitions": partition_info["partitions"],
             "categories": categories,
             "filters": filters,
+            **kwargs,
         }
 
         return parts, stats, common_kwargs
@@ -2259,7 +2363,7 @@ class ArrowLegacyEngine(ArrowDatasetEngine):
     ):
         """Read in a pyarrow table.
 
-        This method is overrides the `ArrowLegacyEngine` implementation.
+        This method is overrides the `ArrowDatasetEngine` implementation.
         """
 
         return _read_table_from_path(
