@@ -7,19 +7,18 @@ from dataclasses import fields, is_dataclass
 
 from tlz import concat, curry, merge, unique
 
-from . import config, threaded
-from .base import (
+from dask import config, threaded
+from dask.base import (
     DaskMethodsMixin,
     dont_optimize,
     is_dask_collection,
     replace_name_in_key,
 )
-from .base import tokenize as _tokenize
-from .context import globalmethod
-from .core import quote
-from .highlevelgraph import HighLevelGraph
-from .optimization import cull
-from .utils import OperatorMethodMixin, apply, ensure_dict, funcname, methodcaller
+from dask.base import tokenize as _tokenize
+from dask.context import globalmethod
+from dask.core import flatten, quote
+from dask.highlevelgraph import HighLevelGraph
+from dask.utils import OperatorMethodMixin, apply, funcname, methodcaller
 
 __all__ = ["Delayed", "delayed"]
 
@@ -207,7 +206,7 @@ def to_task_dask(expr):
     return expr, {}
 
 
-def tokenize(*args, **kwargs):
+def tokenize(*args, pure=None, **kwargs):
     """Mapping function from task -> consistent name.
 
     Parameters
@@ -219,7 +218,6 @@ def tokenize(*args, **kwargs):
         fails, then a unique identifier is used. If False (default), then a
         unique identifier is always used.
     """
-    pure = kwargs.pop("pure", None)
     if pure is None:
         pure = config.get("delayed_pure", False)
 
@@ -472,9 +470,12 @@ def right(method):
 
 
 def optimize(dsk, keys, **kwargs):
-    dsk = ensure_dict(dsk)
-    dsk2, _ = cull(dsk, keys)
-    return dsk2
+    if not isinstance(keys, (list, set)):
+        keys = [keys]
+    if not isinstance(dsk, HighLevelGraph):
+        dsk = HighLevelGraph.from_collections(id(dsk), dsk, dependencies=())
+    dsk = dsk.cull(set(flatten(keys)))
+    return dsk
 
 
 class Delayed(DaskMethodsMixin, OperatorMethodMixin):
@@ -483,13 +484,20 @@ class Delayed(DaskMethodsMixin, OperatorMethodMixin):
     Equivalent to the output from a single key in a dask graph.
     """
 
-    __slots__ = ("_key", "_dask", "_length")
+    __slots__ = ("_key", "_dask", "_length", "_layer")
 
-    def __init__(self, key, dsk, length=None):
+    def __init__(self, key, dsk, length=None, layer=None):
         self._key = key
         self._dask = dsk
         self._length = length
 
+        # NOTE: Layer is used by `to_delayed` in other collections, but not in normal Delayed use
+        self._layer = layer or key
+        if isinstance(dsk, HighLevelGraph) and self._layer not in dsk.layers:
+            raise ValueError(
+                f"Layer {self._layer} not in the HighLevelGraph's layers: {list(dsk.layers)}"
+            )
+
     @property
     def key(self):
         return self._key
@@ -505,12 +513,7 @@ class Delayed(DaskMethodsMixin, OperatorMethodMixin):
         return [self.key]
 
     def __dask_layers__(self):
-        # Delayed objects created with .to_delayed() have exactly
-        # one layer which may have a non-canonical name "delayed-<original name>"
-        if isinstance(self.dask, HighLevelGraph) and len(self.dask.layers) == 1:
-            return tuple(self.dask.layers)
-        else:
-            return (self.key,)
+        return (self._layer,)
 
     def __dask_tokenize__(self):
         return self.key
@@ -526,7 +529,15 @@ class Delayed(DaskMethodsMixin, OperatorMethodMixin):
 
     def _rebuild(self, dsk, *, rename=None):
         key = replace_name_in_key(self.key, rename) if rename else self.key
-        return Delayed(key, dsk, self._length)
+        if isinstance(dsk, HighLevelGraph) and len(dsk.layers) == 1:
+            # FIXME Delayed is currently the only collection type that supports both high- and low-level graphs.
+            # The HLG output of `optimize` will have a layer name that doesn't match `key`.
+            # Remove this when Delayed is HLG-only (because `optimize` will only be passed HLGs, so it won't have
+            # to generate random layer names).
+            layer = next(iter(dsk.layers))
+        else:
+            layer = None
+        return Delayed(key, dsk, self._length, layer=layer)
 
     def __repr__(self):
         return f"Delayed({repr(self.key)})"
@@ -576,12 +587,10 @@ class Delayed(DaskMethodsMixin, OperatorMethodMixin):
             raise TypeError("Delayed objects of unspecified length have no len()")
         return self._length
 
-    def __call__(self, *args, **kwargs):
-        pure = kwargs.pop("pure", None)
-        name = kwargs.pop("dask_key_name", None)
+    def __call__(self, *args, pure=None, dask_key_name=None, **kwargs):
         func = delayed(apply, pure=pure)
-        if name is not None:
-            return func(self, args, kwargs, dask_key_name=name)
+        if dask_key_name is not None:
+            return func(self, args, kwargs, dask_key_name=dask_key_name)
         return func(self, args, kwargs)
 
     def __bool__(self):
@@ -651,6 +660,14 @@ class DelayedLeaf(Delayed):
             self._obj, self._key, args, kwargs, pure=self._pure, nout=self._nout
         )
 
+    @property
+    def __name__(self):
+        return self._obj.__name__
+
+    @property
+    def __doc__(self):
+        return self._obj.__doc__
+
 
 class DelayedAttr(Delayed):
     __slots__ = ("_obj", "_attr")
