@@ -1,35 +1,34 @@
 import os
+import site
 import stat
 import sys
 from collections import OrderedDict
 from contextlib import contextmanager
 
 import pytest
+import yaml
 
 import dask.config
 from dask.config import (
-    update,
-    merge,
-    collect,
-    collect_yaml,
-    collect_env,
-    get,
-    ensure_file,
-    set,
-    config,
-    rename,
-    update_defaults,
-    refresh,
-    expand_environment_variables,
+    _get_paths,
     canonical_name,
-    serialize,
+    collect,
+    collect_env,
+    collect_yaml,
+    config,
     deserialize,
+    ensure_file,
+    expand_environment_variables,
+    get,
+    merge,
+    refresh,
+    rename,
+    serialize,
+    update,
+    update_defaults,
 )
-
 from dask.utils import tmpfile
 
-yaml = pytest.importorskip("yaml")
-
 
 def test_canonical_name():
     c = {"foo-bar": 1, "fizz_buzz": 2}
@@ -137,6 +136,34 @@ def test_collect_yaml_permission_errors(tmpdir, kind):
         assert config == expected
 
 
+def test_collect_yaml_malformed_file(tmpdir):
+    dir_path = str(tmpdir)
+    fil_path = os.path.join(dir_path, "a.yaml")
+
+    with open(fil_path, mode="wb") as f:
+        f.write(b"{")
+
+    with pytest.raises(ValueError) as rec:
+        collect_yaml(paths=[dir_path])
+    assert repr(fil_path) in str(rec.value)
+    assert "is malformed" in str(rec.value)
+    assert "original error message" in str(rec.value)
+
+
+def test_collect_yaml_no_top_level_dict(tmpdir):
+    dir_path = str(tmpdir)
+    fil_path = os.path.join(dir_path, "a.yaml")
+
+    with open(fil_path, mode="wb") as f:
+        f.write(b"[1234]")
+
+    with pytest.raises(ValueError) as rec:
+        collect_yaml(paths=[dir_path])
+    assert repr(fil_path) in str(rec.value)
+    assert "is malformed" in str(rec.value)
+    assert "must have a dict" in str(rec.value)
+
+
 def test_env():
     env = {
         "DASK_A_B": "123",
@@ -180,13 +207,10 @@ def test_collect():
             assert config == expected
 
 
-def test_collect_env_none():
-    os.environ["DASK_FOO"] = "bar"
-    try:
-        config = collect([])
-        assert config == {"foo": "bar"}
-    finally:
-        del os.environ["DASK_FOO"]
+def test_collect_env_none(monkeypatch):
+    monkeypatch.setenv("DASK_FOO", "bar")
+    config = collect([])
+    assert config == {"foo": "bar"}
 
 
 def test_get():
@@ -241,47 +265,47 @@ def test_ensure_file(tmpdir):
 
 
 def test_set():
-    with set(abc=123):
+    with dask.config.set(abc=123):
         assert config["abc"] == 123
-        with set(abc=456):
+        with dask.config.set(abc=456):
             assert config["abc"] == 456
         assert config["abc"] == 123
 
     assert "abc" not in config
 
-    with set({"abc": 123}):
+    with dask.config.set({"abc": 123}):
         assert config["abc"] == 123
     assert "abc" not in config
 
-    with set({"abc.x": 1, "abc.y": 2, "abc.z.a": 3}):
+    with dask.config.set({"abc.x": 1, "abc.y": 2, "abc.z.a": 3}):
         assert config["abc"] == {"x": 1, "y": 2, "z": {"a": 3}}
     assert "abc" not in config
 
     d = {}
-    set({"abc.x": 123}, config=d)
+    dask.config.set({"abc.x": 123}, config=d)
     assert d["abc"]["x"] == 123
 
 
 def test_set_kwargs():
-    with set(foo__bar=1, foo__baz=2):
+    with dask.config.set(foo__bar=1, foo__baz=2):
         assert config["foo"] == {"bar": 1, "baz": 2}
     assert "foo" not in config
 
     # Mix kwargs and dict, kwargs override
-    with set({"foo.bar": 1, "foo.baz": 2}, foo__buzz=3, foo__bar=4):
+    with dask.config.set({"foo.bar": 1, "foo.baz": 2}, foo__buzz=3, foo__bar=4):
         assert config["foo"] == {"bar": 4, "baz": 2, "buzz": 3}
     assert "foo" not in config
 
     # Mix kwargs and nested dict, kwargs override
-    with set({"foo": {"bar": 1, "baz": 2}}, foo__buzz=3, foo__bar=4):
+    with dask.config.set({"foo": {"bar": 1, "baz": 2}}, foo__buzz=3, foo__bar=4):
         assert config["foo"] == {"bar": 4, "baz": 2, "buzz": 3}
     assert "foo" not in config
 
 
 def test_set_nested():
-    with set({"abc": {"x": 123}}):
+    with dask.config.set({"abc": {"x": 123}}):
         assert config["abc"] == {"x": 123}
-        with set({"abc.y": 456}):
+        with dask.config.set({"abc.y": 456}):
             assert config["abc"] == {"x": 123, "y": 456}
         assert config["abc"] == {"x": 123}
     assert "abc" not in config
@@ -290,8 +314,8 @@ def test_set_nested():
 def test_set_hard_to_copyables():
     import threading
 
-    with set(x=threading.Lock()):
-        with set(y=1):
+    with dask.config.set(x=threading.Lock()):
+        with dask.config.set(y=1):
             pass
 
 
@@ -367,12 +391,9 @@ def test_refresh():
         ({"a": "A", "b": [1, "2", "$FOO"]}, {"a": "A", "b": [1, "2", "foo"]}),
     ],
 )
-def test_expand_environment_variables(inp, out):
-    try:
-        os.environ["FOO"] = "foo"
-        assert expand_environment_variables(inp) == out
-    finally:
-        del os.environ["FOO"]
+def test_expand_environment_variables(monkeypatch, inp, out):
+    monkeypatch.setenv("FOO", "foo")
+    assert expand_environment_variables(inp) == out
 
 
 def test_env_var_canonical_name(monkeypatch):
@@ -510,3 +531,45 @@ def test_config_inheritance():
         {"DASK_INTERNAL_INHERIT_CONFIG": serialize({"array": {"svg": {"size": 150}}})}
     )
     assert dask.config.get("array.svg.size", config=config) == 150
+
+
+def test__get_paths(monkeypatch):
+    # These settings are used by Dask's config system. We temporarily
+    # remove them to avoid interference from the machine where tests
+    # are being run.
+    monkeypatch.delenv("DASK_CONFIG", raising=False)
+    monkeypatch.delenv("DASK_ROOT_CONFIG", raising=False)
+    monkeypatch.setattr(site, "PREFIXES", [])
+
+    expected = [
+        "/etc/dask",
+        os.path.join(sys.prefix, "etc", "dask"),
+        os.path.join(os.path.expanduser("~"), ".config", "dask"),
+    ]
+    paths = _get_paths()
+    assert paths == expected
+    assert len(paths) == len(set(paths))  # No duplicate paths
+
+    with monkeypatch.context() as m:
+        m.setenv("DASK_CONFIG", "foo-bar")
+        paths = _get_paths()
+        assert paths == expected + ["foo-bar"]
+        assert len(paths) == len(set(paths))
+
+    with monkeypatch.context() as m:
+        m.setenv("DASK_ROOT_CONFIG", "foo-bar")
+        paths = _get_paths()
+        assert paths == ["foo-bar"] + expected[1:]
+        assert len(paths) == len(set(paths))
+
+    with monkeypatch.context() as m:
+        prefix = os.path.join("include", "this", "path")
+        m.setattr(site, "PREFIXES", site.PREFIXES + [prefix])
+        paths = _get_paths()
+        assert os.path.join(prefix, "etc", "dask") in paths
+        assert len(paths) == len(set(paths))
+
+
+def test_default_search_paths():
+    # Ensure _get_paths() is used for default paths
+    assert dask.config.paths == _get_paths()
