@@ -15,6 +15,7 @@ from dask.utils import (
     SerializableLock,
     _deprecated,
     asciitable,
+    cached_cumsum,
     derived_from,
     ensure_dict,
     extra_titles,
@@ -22,14 +23,12 @@ from dask.utils import (
     funcname,
     getargspec,
     has_keyword,
-    ignoring,
     is_arraylike,
     itemgetter,
     iter_chunks,
     memory_repr,
     methodcaller,
     ndeepmap,
-    noop_context,
     parse_bytes,
     parse_timedelta,
     partial_by_order,
@@ -219,7 +218,7 @@ def test_random_state_data():
 
 def test_memory_repr():
     for power, mem_repr in enumerate(["1.0 bytes", "1.0 KB", "1.0 MB", "1.0 GB"]):
-        assert memory_repr(1024 ** power) == mem_repr
+        assert memory_repr(1024**power) == mem_repr
 
 
 def test_method_caller():
@@ -475,6 +474,10 @@ def test_itemgetter():
     assert g2(data) == 2
     assert g2.index == 1
 
+    assert itemgetter(1) == itemgetter(1)
+    assert itemgetter(1) != itemgetter(2)
+    assert itemgetter(1) != 123
+
 
 def test_partial_by_order():
     assert partial_by_order(5, function=operator.add, other=[(1, 20)]) == 25
@@ -554,6 +557,8 @@ def test_derived_from_dask_dataframe():
     assert "not supported" in axis_arg.lower()
     assert "dask" in axis_arg.lower()
 
+    assert "Object with missing values filled" in dd.DataFrame.ffill.__doc__
+
 
 def test_parse_bytes():
     assert parse_bytes("100") == 100
@@ -562,7 +567,7 @@ def test_parse_bytes():
     assert parse_bytes("5kB") == 5000
     assert parse_bytes("5.4 kB") == 5400
     assert parse_bytes("1kiB") == 1024
-    assert parse_bytes("1Mi") == 2 ** 20
+    assert parse_bytes("1Mi") == 2**20
     assert parse_bytes("1e6") == 1000000
     assert parse_bytes("1e6 kB") == 1000000000
     assert parse_bytes("MB") == 1000000
@@ -682,15 +687,15 @@ def test_stringify_collection_keys():
         (0, "0 B"),
         (920, "920 B"),
         (930, "0.91 kiB"),
-        (921.23 * 2 ** 10, "921.23 kiB"),
-        (931.23 * 2 ** 10, "0.91 MiB"),
-        (921.23 * 2 ** 20, "921.23 MiB"),
-        (931.23 * 2 ** 20, "0.91 GiB"),
-        (921.23 * 2 ** 30, "921.23 GiB"),
-        (931.23 * 2 ** 30, "0.91 TiB"),
-        (921.23 * 2 ** 40, "921.23 TiB"),
-        (931.23 * 2 ** 40, "0.91 PiB"),
-        (2 ** 60, "1024.00 PiB"),
+        (921.23 * 2**10, "921.23 kiB"),
+        (931.23 * 2**10, "0.91 MiB"),
+        (921.23 * 2**20, "921.23 MiB"),
+        (931.23 * 2**20, "0.91 GiB"),
+        (921.23 * 2**30, "921.23 GiB"),
+        (931.23 * 2**30, "0.91 TiB"),
+        (921.23 * 2**40, "921.23 TiB"),
+        (931.23 * 2**40, "0.91 PiB"),
+        (2**60, "1024.00 PiB"),
     ],
 )
 def test_format_bytes(n, expect):
@@ -720,6 +725,15 @@ def test_deprecated_version():
         assert foo() == "bar"
 
 
+def test_deprecated_after_version():
+    @_deprecated(after_version="1.2.3")
+    def foo():
+        return "bar"
+
+    with pytest.warns(FutureWarning, match="deprecated after version 1.2.3"):
+        assert foo() == "bar"
+
+
 def test_deprecated_category():
     @_deprecated(category=DeprecationWarning)
     def foo():
@@ -741,18 +755,6 @@ def test_deprecated_message():
     assert str(record[0].message) == "woohoo"
 
 
-def test_ignoring_deprecated():
-    with pytest.warns(FutureWarning, match="contextlib.suppress"):
-        with ignoring(ValueError):
-            pass
-
-
-def test_noop_context_deprecated():
-    with pytest.warns(FutureWarning, match="contextlib.nullcontext"):
-        with noop_context():
-            pass
-
-
 def test_typename():
     assert typename(HighLevelGraph) == "dask.highlevelgraph.HighLevelGraph"
     assert typename(HighLevelGraph, short=True) == "dask.HighLevelGraph"
@@ -765,3 +767,27 @@ class MyType:
 def test_typename_on_instances():
     instance = MyType()
     assert typename(instance) == typename(MyType)
+
+
+def test_cached_cumsum():
+    a = (1, 2, 3, 4)
+    x = cached_cumsum(a)
+    y = cached_cumsum(a, initial_zero=True)
+    assert x == (1, 3, 6, 10)
+    assert y == (0, 1, 3, 6, 10)
+
+
+def test_cached_cumsum_nan():
+    np = pytest.importorskip("numpy")
+    a = (1, np.nan, 3)
+    x = cached_cumsum(a)
+    y = cached_cumsum(a, initial_zero=True)
+    np.testing.assert_equal(x, (1, np.nan, np.nan))
+    np.testing.assert_equal(y, (0, 1, np.nan, np.nan))
+
+
+def test_cached_cumsum_non_tuple():
+    a = [1, 2, 3]
+    assert cached_cumsum(a) == (1, 3, 6)
+    a[1] = 4
+    assert cached_cumsum(a) == (1, 5, 8)
